
(function($) {

  var collapsed = {};

  /**
   * Behavior on our custom field to erase the calculated values when someone changes the value of the test fields.
   */
  Drupal.behaviors.cgpaCalculatorTermSet = function(context) {
    $('.cgpa-calculator-term-set-table:not(.cgpa-calculator-term-set-table-processed)', context)
      .addClass('cgpa-calculator-term-set-table-processed')
      .each(function() {

        // If the input changes, reset the row's calculated values.
        $(this).find('tr:has(td) input[type="text"]').bind('change', function() {
          $(this).parents('tr:first').find('.cgpa-calculator-term-set-info').text('');
        });


        /**
         * Tweak fieldset expansion so if we replace it with AHAH, it will expand again on view.
         */
        (function() {
          var fieldset = $(this).parents('fieldset:first');
          var col = fieldset.is('.collapsed');
          var id = fieldset.parents('div:first').attr('id');

          if (typeof collapsed[id] === 'undefined') {
            collapsed[id] = col;
          }
          else if (!collapsed[id] && col) {
            Drupal.toggleFieldset(fieldset);
          }
        })();
      });
  };

  /**
   * Behavior to automatically submit the exposed form when an option is selected.
   */
  Drupal.behaviors.cgpaCalculatorAdmissionsView = function(context) {
    $('#views-exposed-form-cgpa-calculator-admissions-view-page-1:not(.cgpa-calculator-admissions-view-processed)', context)
      .addClass('cgpa-calculator-admissions-view-processed')
      .each(function() {
        var form = $(this);

        // Watch the change event, and submit
        form.find('select').change(function() {
          form.submit();
        });

        // Hide submit button
        form.find('.form-submit').hide();
      });
  };

  /**
   * Behavior to capture 'enter' key presses, to force them to press proper submit button.
   * Otherwise pressing enter will trigger first submit button in form, adding items to field.
   */
  Drupal.behaviors.cgpaCalculatorSubmitEnter = function(context) {
    $('.cgpa-calculator-node-form:not(.cgpa-calculator-submit-enter-processed)', context)
      .addClass('cgpa-calculator-submit-enter-processed')
      .keypress(function(e) {
        if (e.which == 13) {
          e.preventDefault();
          $(this).find('#edit-submit').click();
        }
      });
  };
  
  /**
   * Behavior to reset form fields per term.
   * Uses Reset term button to clear fields.
   */
   Drupal.behaviors.cgpaCalculatorResetTerm = function(context) {
      if($('.cgpa-calculator-node-form')) { //possibly need a better criterion for this behavior to fire...
          var submits = $('.form-submit[value="Reset Term"]'); //gets all the submits that will reset a term...
          submits.click(function(e){ //add listener to click event for all submits
              e.preventDefault();
              resetTermFields(this.id);
          });
          var resetTermFields = function(id){ //accepts id of reset button clicked
              var fIds = id;
              var yearNo = fIds.split('year-')[1].split('-')[0]; //get the year no
              var termNo = fIds.split('year-')[1].split('-')[1]; //get the term no
              var yearTerms = $('#field-year-' + yearNo + '-items').find('.form-submit[value="Reset Term"]'); //get ther number of terms (from the number of resets buttons) in this year
              var yearItems = $('#field-year-' + yearNo + '-items').find('.cgpa-calculator-term-set-mark input, .cgpa-calculator-term-set-credits input'); //get all of the items elegible for reset by year
              var getItems = yearItems.length/yearTerms.length; //get the number of items/term to determine slice size
              yearItems = yearItems.slice(termNo * getItems, (termNo + 1 ) * getItems); //slice up the year array into terms
              yearItems.push($('#edit-field-year-' + yearNo + '-' + termNo + '-term')[0]); //adding that extra element to the array (outside of the other markup)
              yearItems.val(''); //blow away values
          };   
      }
   };

 })(jQuery);
